#ifndef STROGE_H
#define STROGE_H

#include <sqlite3.h>

#ifdef  __cplusplus
extern "C" {
#endif

#define _STR(s)	#s
#define DEF2STR(s)	_STR(s)

#ifdef BIO_DB_DIR
	#define DB_DIR	DEF2STR(BIO_DB_DIR)
#else
	#define DB_DIR	"/var/lib/biometric-auth/"
#endif

#ifdef BIO_DB_NAME
	#define DB_NAME	DEF2STR(BIO_DB_NAME)
#else
	#define DB_NAME	"biometric.db"
#endif


#define DB_PATH				DB_DIR DB_NAME
#define TABLE_EIGEN_INFO	"EIGEN_INFO"

/**
* @brief 特征采样数据链表
* 记录单个特征部位的单次特征采样值的链表
* @par 修改日志
* JianglinXuan于2017-05-17创建
*/
typedef struct feature_sample_t feature_sample;
struct feature_sample_t {
	int no;				/** 采样的序号 */
	char *data;			/** 采样的数据 */
	feature_sample *next;	/** 链表中下一个采样值 */
};

/**
* @brief 各特征部位信息链表
* 记录单个特征部位详细信息的链表
* @par 修改日志
* JianglinXuan于2017-05-17创建
*/
typedef struct feature_info_t feature_info;
struct feature_info_t {
	int	uid;							/** 特征部位所属的用户ID */
	int biotype;						/** 特征部位的类型 */
	char *driver;						/** 特征对应的驱动名 */
	int	index;							/** 特征部位的索引值 */
	char *index_name;					/** 特征部位的名称，由用户设置 */
	feature_sample *sample;				/** 特征部位的采样值链表 */
	feature_info *next;	/** 链表中的下一个特征部位信息 */
};


/* 内部函数，用于输出特征信息结构体 */
void print_feature_info(feature_info * info_list);

/* 连接数据库 */
sqlite3 *bio_sto_connect_db();

/* 断开数据库连接 */
void bio_sto_disconnect_db(sqlite3 *db);

/* 创建单项采样数据 */
feature_sample * bio_sto_new_feature_sample(int no, char * data);

/* 回收单项采样数据 */
void bio_sto_free_feature_sample(feature_sample * sample);

/* 回收采样数据链表 */
void bio_sto_free_feature_sample_list(feature_sample * sample_list);

/* 回收单项特征数据 */
void bio_sto_free_feature_info(feature_info * info);

/* 回收特征数据链表 */
void bio_sto_free_feature_info_list(feature_info * info_list);

/* 创建单项特征数据 */
feature_info * bio_sto_new_feature_info(int	uid,
														int biotype,
														char *driver,
														int	index,
														char *index_name);
/* 获取特征信息链表 */
feature_info *bio_sto_get_feature_info(sqlite3 *db,
													   int uid,
													   int biotype,
													   char *driver,
													   int index_start,
													   int index_end);

/* 保存特征信息链表 */
int bio_sto_set_feature_info(sqlite3 *db,
									 feature_info * info_list);

/* 删除特征信息 */
int bio_sto_clean_feature_info(sqlite3 *db, int uid, int biotype,
									   char *driver, int index_start,
									   int index_end);

char * bio_sto_new_str(char * old_str);

#ifdef  __cplusplus
}
#endif

#endif // STROGE_H
